# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
from unittest import mock

import pytest
import ruamel.yaml as yaml

from qdk.chemistry.solvers.nwchem import create_input_deck, parse_nwchem_output


@pytest.fixture()
def test_deck():
    return """
start HHO_test

echo
memory stack 1000 mb heap 100 mb global 1000 mb noverify

geometry units au
symmetry c1
O 0.002 0.398 0.0
H 0.762 -0.203 0.0
H -0.764 -0.195 0.0
end

basis
* library sto-3g
end



scf
thresh 1.0e-08
tol2e 1e-09
singlet
rhf
maxiter 200

end

tce
ccsd
2eorb
2emet 13
tilesize 20

thresh 1.0e-08
end

set tce:print_integrals T
set tce:qorb 7
set tce:qela 3
set tce:qelb 3

task tce energy
"""


def test_nwchem(geometry, h2o, test_deck):
    mol_name = "HHO_test"
    with mock.patch("qdk.chemistry.geometry.Geometry.from_mol") as _m:
        _m.return_value = geometry

        nw_chem_input = create_input_deck(
            mol=h2o,
            mol_name=mol_name,
            num_active_orbitals=7
        )

    assert nw_chem_input == test_deck


def test_nwchem_pass_geometry(geometry, h2o, test_deck):
    mol_name = "HHO_test"
    nw_chem_input = create_input_deck(
        mol_name=mol_name,
        geometry=geometry,
        num_active_orbitals=7,
        mol=h2o
    )

    assert nw_chem_input == test_deck


def test_parse_nwchem_output(caffeine_nw, caffeine_output):
    assert parse_nwchem_output(caffeine_nw, caffeine_output) == {
        'number of atoms': 24,
        'number of orbitals': 80,
        'SCF energy': -627.628748906485,
        'CCSD correlation energy': -0.002197738334726,
        'geometry snapshot': []
    }



# Copyright (c) Microsoft Corporation.
# Copyright (c) 2018 Jensen Group
# Licensed under the MIT License.

"""
Module for generating rdkit molobj/smiles/molecular graph from free atoms
Implementation by Jan H. Jensen, based on the paper
    Yeonjoon Kim and Woo Youn Kim
    "Universal Structure Conversion Method for Organic Molecules: From Atomic Connectivity
    to Three-Dimensional Geometry"
    Bull. Korean Chem. Soc. 2015, Vol. 36, 1769-1777
    DOI: 10.1002/bkcs.10334
"""
from qdk.chemistry._xyz2mol.ac import xyz2AC, AC2mol
from qdk.chemistry._xyz2mol.util import chiral_stereo_check


def xyz2mol(
    atoms, 
    coordinates,
    charge=0,
    allow_charged_fragments=True,
    use_graph=True,
    use_huckel=False,
    embed_chiral=True
):
    """
    Generate a rdkit molobj from atoms, coordinates and a total_charge.
    args:
        atoms - list of atom types (int)
        coordinates - 3xN Cartesian coordinates
        charge - total charge of the system (default: 0)
    optional:
        allow_charged_fragments - alternatively radicals are made
        use_graph - use graph (networkx)
        use_huckel - Use Huckel method for atom connectivity prediction
        embed_chiral - embed chiral information to the molecule
    returns:
        mols - list of rdkit molobjects
    """

    # Get atom connectivity (AC) matrix, list of atomic numbers, molecular charge,
    # and mol object with no connectivity information
    AC, mol = xyz2AC(atoms, coordinates, charge, use_huckel=use_huckel)

    # Convert AC to bond order matrix and add connectivity and charge info to
    # mol object
    new_mols = AC2mol(mol, AC, atoms, charge,
        allow_charged_fragments=allow_charged_fragments,
        use_graph=use_graph)

    # Check for stereocenters and chiral centers
    if embed_chiral:
        for new_mol in new_mols:
            chiral_stereo_check(new_mol)

    return new_mols



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.

import enum
from typing import Union, TYPE_CHECKING

from qdk.chemistry.geometry import Geometry

if TYPE_CHECKING:
    from rdkit.Chem.AllChem import Mol


class Basis(enum.Enum):
    gaussian = "3-21G"
    pople = "6-31G"
    dunning = "cc-pVDZ"
    karlsruhe = "def2-SVP"


PSI4_TEMPLATE = """
memory {N} GB

molecule {name} {{
symmetry {symmetry}
{charge} {spin}
  {geometry}
}}

set {{
  basis {basis}
  scf_type {scf_type}
  reference {reference}
  d_convergence 1e-8
  e_convergence 1e-8
}}

{method_section}
"""

PSI4_TEMPLATE_ENERGY = "e = {driver}('{method}')"


PSI4_TEMPLATE_FCIDUMP = """
e, wfn = {driver}('SCF', return_wfn=True)
{optional_energy}
fcidump(wfn, fname='fcidump', oe_ints=['EIGENVALUES'])
clean()
"""


def create_input_deck(
    mol: "Mol",
    mol_name: str = "",
    geometry: Union[str, Geometry] = None,
    charge: int = 0,
    spin: str = 1,
    basis: str = "3-21G",
    symmetry: str = "C1",
    method: str = "SCF",
    driver: str = "energy",
    scf_type: str = "PK",
    memory_in_gb: int = 1,
    reference: str = "rhf"
) -> str:
    """Create Psi4 input formatted string

    :param mol: RDKit Molecule object
    :type mol: Mol
    :param mol_name: Molecule name, defaults to ""
    :type mol_name: str, optional
    :param geometry: Geometry object, defaults to None
    :type geometry: Union[str, Geometry], optional
    :param charge: Molecule charge, defaults to 0
    :type charge: int, optional
    :param spin: Molecule spin, defaults to 1
    :type spin: str, optional
    :param basis: Molecule basis, defaults to "3-21G"
    :type basis: str, optional
    :param symmetry: Molecule symmetry
    (example options: "C1", "C2v", "D6h", "Cs"),
         defaults to "C1". When requesting Broombridge, make sure to set this
         to "C1" because Broombridge does not support group symmetry
    :type symmetry: str, optional
    :param method: Molecule method (options: "SCF", "HF", "DFT", "CC2",
        "CCSD", "CASSCF", "MP2"), defaults to "SCF"
    :type method: str, optional
    :param driver: Driver method ("energy" or "optimize"), defaults to "energy"
    :type driver: str, optional
    :param scf_type: SCF solver type ("DIRECT", "DF", "PK",
        "OUT_OF_CORE" or "PS"), defaults to "PK"
    :type scf_type: str, optional
    :param memory_in_gb: Memory used in GB
    :type memory_in_gb: int
    :param reference: Reference, defaults to "rhf"
    :type reference: str, optional
    :return: Psi4 input-formatted string
    :rtype: str
    """
    assert basis in Basis._value2member_map_, f"Unknown basis: {basis}. \
        Please choose from the following: {Basis._value2member_map_.values()}"
    optional_energy = PSI4_TEMPLATE_ENERGY.format(
        driver=driver, method=method) if method.upper() != "SCF" else ""
    template = PSI4_TEMPLATE_FCIDUMP.format(
        driver=driver, optional_energy=optional_energy)
    method_section = template.format(driver=driver, method=method)

    return PSI4_TEMPLATE.format(
        N=memory_in_gb,
        name=mol_name,
        symmetry=symmetry,
        charge=charge,
        spin=spin,
        geometry=geometry,
        basis=basis,
        scf_type=scf_type,
        reference=reference,
        method_section=method_section
    )



#!/bin/env python
# -*- coding: utf-8 -*-
##
# setup.py: Installs QDK-Python: Python tools for the
# Microsoft Quantum Development Kit
##
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
##
import setuptools
import os

version = os.environ.get('PYTHON_VERSION', '0.0.0.1')
is_conda = bool(os.environ.get('CONDA_BUILD', False))

with open("./README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="qdk",
    version=version,
    author="Microsoft",
    author_email="que-contacts@microsoft.com",
    description="Quantum Development Kit",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/microsoft/qdk-python",
    packages=setuptools.find_namespace_packages(include=["qdk*"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'qsharp',
        'jupyter_jsmol',
        'networkx',
        'varname',
        'ruamel-yaml',
        'basis_set_exchange',
        'jupyter_nbextensions_configurator',
        'pygments>=2.7.4',
        'ipython>=5.11.0',
        'ipywidgets==8.0.4' # version 8.0.5 causes JsmolView to break. See: https://github.com/fekad/jupyter-jsmol/issues/58
    ]
)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
import pytest
from unittest import mock

from qdk.chemistry.solvers.psi4 import create_input_deck


@pytest.fixture()
def test_deck():
    return """
memory 1 GB

molecule  {
symmetry C1
0 1
  None
}

set {
  basis 3-21G
  scf_type PK
  reference rhf
  d_convergence 1e-8
  e_convergence 1e-8
}


e, wfn = energy('SCF', return_wfn=True)
e = energy('HF')
fcidump(wfn, fname='fcidump', oe_ints=['EIGENVALUES'])
clean()

"""


def test_psi4(geometry, h2o, test_deck):
    with mock.patch("qdk.chemistry.geometry.Geometry.from_mol") as _m:
        _m.return_value = geometry

        psi4_input = create_input_deck(
            mol=h2o,
            method="HF",
            driver="energy"
        )

    assert psi4_input == test_deck
